# -*- coding:utf-8 -*-

# package information.
INFO = dict(
  name        = "rita",
  description = "Python Web Application Framework",
  author      = "Takayoshi Urushio",
  author_email = "uru@craft-works.jp",
  license     = "MIT License",
  url         = "https://bitbucket.com/urushio",
  classifiers = [
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "License :: OSI Approved :: MIT License"
  ]
)
